package org.eclipse.contribution.xref.internal.ui;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.contribution.xref.internal.core.XReferenceAdapterFactory;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * The main plugin class to be used in the desktop.
 */
public class XReferenceUIPlugin extends AbstractUIPlugin {
	
	//The shared instance.
	private static XReferenceUIPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	/**
	 * The constructor.
	 */
	public XReferenceUIPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		plugin = this;
		try {
			resourceBundle= ResourceBundle.getBundle("org.eclipse.contribution.xref.internal.ui.XReferenceUIPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
		
		registerAdapter();
	}

	/**
	 * Returns the shared instance.
	 */
	public static XReferenceUIPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle= XReferenceUIPlugin.getDefault().getResourceBundle();
		try {
			return (bundle!=null ? bundle.getString(key) : key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/*
	 * Helper method to register the required resources with 
	 * a XReferenceAdapterFactory.
	 *
	 */	
	private void registerAdapter() {
		XReferenceAdapterFactory xra = new XReferenceAdapterFactory();
		Platform.getAdapterManager().registerAdapters(xra, IJavaElement.class);	
		Platform.getAdapterManager().registerAdapters(xra, IResource.class);	
	}

	private static Image evaluateImage = null;
	public Image getEvaluateImage() {
		if (evaluateImage == null) {
			try {
				URL pluginInstallURL = getDescriptor().getInstallURL();
				ImageDescriptor d = ImageDescriptor.createFromURL(
						new URL(pluginInstallURL,"icons/updates_obj.gif"));
				evaluateImage = d.createImage();
			} catch (MalformedURLException mex) {
				System.err.println("Couldn't create evaluateImage");
			}
		}
		return evaluateImage;		
	}
	
	private static Image relImage = null;
	public Image getXReferenceImage() {
		if (relImage == null) {
			try {
				URL pluginInstallURL = getDescriptor().getInstallURL();
				ImageDescriptor d = ImageDescriptor.createFromURL(
						new URL(pluginInstallURL,"icons/forward_nav.gif"));
				relImage = d.createImage();
			} catch (MalformedURLException mex) {
				System.err.println("Couldn't create relImage");
			}
		}
		return relImage;		
	}
	
}
