/*
 * Created on 18-Jan-2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package org.aspect.ajdt.internal.core;

import org.aspectj.ajdt.core.IAspectJElement;
import org.aspectj.ajdt.core.IPointcut;
import org.eclipse.jdt.internal.core.Assert;
import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.Signature;
import org.aspectj.ajdt.core.jdom.IAjDOMNode;
import org.aspectj.ajdt.core.jdom.IDOMPointcut;
import org.eclipse.jdt.core.jdom.IDOMNode;
import org.eclipse.jdt.internal.core.CompilationUnit;
import org.eclipse.jdt.internal.core.JavaElement;
import org.eclipse.jdt.internal.core.Member;
import org.eclipse.jdt.internal.core.util.Util;

/**
 * @author colyer
 *
 * To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class SourcePointcut extends Member implements IPointcut {

	/**
	 * The parameter type signatures of the pointcut - stored locally
	 * to perform equality test. <code>null</code> indicates no
	 * parameters.
	 */
	protected String[] fParameterTypes;

	/**
	 * An empty list of Strings
	 */
	protected static final String[] fgEmptyList= new String[] {};
	
	public SourcePointcut(JavaElement parent, String name, String[] parameterTypes) {
		super(parent, name);
		Assert.isTrue(name.indexOf('.') == -1);
		if (parameterTypes == null) {
			fParameterTypes= fgEmptyList;
		} else {
			fParameterTypes= parameterTypes;
		}
	}
	public boolean equals(Object o) {
		if (!(o instanceof SourcePointcut)) return false;
		return super.equals(o) && Util.equalArraysOrNull(fParameterTypes, ((SourcePointcut)o).fParameterTypes);
	}
	/**
	 * @see JavaElement#equalsDOMNode
	 */
	protected boolean equalsDOMNode(IDOMNode node) {
		if (node.getNodeType() == IAjDOMNode.POINTCUT) {
			IDOMPointcut p = (IDOMPointcut)node;
			return super.equalsDOMNode(node) && signatureEquals(p);
		} else {
			return false;
		}
	}
	/**
	 * @see IJavaElement
	 */
	public int getElementType() {
		return IAspectJElement.POINTCUT;
	}
	
	/**
	 * @see JavaElement#getHandleMemento()
	 */
	public String getHandleMemento() {
		StringBuffer buff = new StringBuffer(((JavaElement) getParent()).getHandleMemento());
		buff.append(getHandleMementoDelimiter());
		buff.append(getElementName());
		for (int i = 0; i < fParameterTypes.length; i++) {
			buff.append(getHandleMementoDelimiter());
			buff.append(fParameterTypes[i]);
		}
		if (this.occurrenceCount > 1) {
			buff.append(JEM_COUNT);
			buff.append(this.occurrenceCount);
		}
		return buff.toString();
	}
	/**
	 * @see JavaElement#getHandleMemento()
	 */
	protected char getHandleMementoDelimiter() {
		return AspectJElement.AJEM_POINTCUT;
	}
	/**
	 * @see IPointcut
	 */
	public int getNumberOfParameters() {
		return fParameterTypes == null ? 0 : fParameterTypes.length;
	}
	/**
	 * @see IPointcut
	 */
	public String[] getParameterNames() throws JavaModelException {
		SourcePointcutElementInfo info = (SourcePointcutElementInfo) getElementInfo();
		char[][] names= info.getArgumentNames();
		if (names == null || names.length == 0) {
			return fgEmptyList;
		}
		String[] strings= new String[names.length];
		for (int i= 0; i < names.length; i++) {
			strings[i]= new String(names[i]);
		}
		return strings;
	}
	/**
	 * @see IMethod
	 */
	public String[] getParameterTypes() {
		return fParameterTypes;
	}
	/*
	 * @see JavaElement#getPrimaryElement(boolean)
	 */
	public IJavaElement getPrimaryElement(boolean checkOwner) {
		if (checkOwner) {
			CompilationUnit cu = (CompilationUnit)getAncestor(COMPILATION_UNIT);
			if (cu.isPrimary()) return this;
		}
		IJavaElement primaryParent = this.parent.getPrimaryElement(false);
		return ((IType)primaryParent).getMethod(this.name, fParameterTypes);
	}
	/**
	 * @see IMethod
	 */
	public String getSignature() throws JavaModelException {
		SourcePointcutElementInfo info = (SourcePointcutElementInfo) getElementInfo();
		return info.getSignature();
	}
	/**
	 * @see IMethod#isSimilar(IMethod)
	 */
	public boolean isSimilar(IPointcut method) {
		return 
		this.areSimilarMethods(
				this.getElementName(), this.getParameterTypes(),
				method.getElementName(), method.getParameterTypes(),
				null);
	}

	/**
	 */
	public String readableName() {

		StringBuffer buffer = new StringBuffer(super.readableName());
		buffer.append('(');
		String[] parameterTypes = this.getParameterTypes();
		int length;
		if (parameterTypes != null && (length = parameterTypes.length) > 0) {
			for (int i = 0; i < length; i++) {
				buffer.append(Signature.toString(parameterTypes[i]));
				if (i < length - 1) {
					buffer.append(", "); //$NON-NLS-1$
				}
			}
		}
		buffer.append(')');
		return buffer.toString();
	}
	/**
	 * Returns <code>true</code> if the signature of this <code>SourceMethod</code> matches that of the given
	 * <code>IDOMMethod</code>, otherwise <code>false</code>. 
	 */
	protected boolean signatureEquals(IDOMPointcut pcut) {
		String[] otherTypes= pcut.getParameterTypes();
		String[] types= getParameterTypes();
		boolean ok= true;

		// ensure the number of parameters match
		if (otherTypes == null || otherTypes.length == 0) {
			ok= (types == null || types.length == 0);
		} else if (types != null) {
			ok= (otherTypes.length == types.length);
		} else {
			return false;
		}

		// ensure the parameter type signatures match
		if (ok) {
			if (types != null) {
				int i;
				for (i= 0; i < types.length; i++) {
					String otherType= Signature.createTypeSignature(otherTypes[i].toCharArray(), false);
					if (!types[i].equals(otherType)) {
						ok= false;
						break;
					}
				}
			}
		}

		return ok;
	}
	/**
	 * @private Debugging purposes
	 */
	protected void toStringInfo(int tab, StringBuffer buffer, Object info) {
		buffer.append(this.tabString(tab));
		if (info == null) {
			buffer.append(getElementName());
			toStringParameters(buffer);
			buffer.append(" (not open)"); //$NON-NLS-1$
		} else if (info == NO_INFO) {
			buffer.append(getElementName());
			toStringParameters(buffer);
		} else {
			try {
				if (Flags.isStatic(this.getFlags())) {
					buffer.append("static "); //$NON-NLS-1$
				}
				buffer.append(this.getElementName());
				toStringParameters(buffer);
			} catch (JavaModelException e) {
				buffer.append("<JavaModelException in toString of " + getElementName()); //$NON-NLS-1$
			}
		}
	}
	private void toStringParameters(StringBuffer buffer) {
		buffer.append('(');
		String[] parameterTypes = this.getParameterTypes();
		int length;
		if (parameterTypes != null && (length = parameterTypes.length) > 0) {
			for (int i = 0; i < length; i++) {
				buffer.append(Signature.toString(parameterTypes[i]));
				if (i < length - 1) {
					buffer.append(", "); //$NON-NLS-1$
				}
			}
		}
		buffer.append(')');
	}

}
