/*
 * Created on 20-Jan-2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package org.aspectj.ajdt.internal.core.builder;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import org.aspectj.ajdt.internal.compiler.AjCompiler;
import org.aspectj.bridge.AbortException;
import org.aspectj.bridge.IMessage;
import org.aspectj.bridge.IMessageHandler;
import org.aspectj.bridge.IProgressListener;
import org.aspectj.bridge.IMessage.Kind;
import org.aspectj.weaver.bcel.UnwovenClassFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IJavaModelMarker;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.compiler.ClassFile;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.Compiler;
import org.eclipse.jdt.internal.compiler.DefaultErrorHandlingPolicies;
import org.eclipse.jdt.internal.core.builder.BatchImageBuilder;
import org.eclipse.jdt.internal.core.builder.JavaBuilder;
import org.eclipse.jdt.internal.core.builder.ProblemFactory;


/**
 * @author colyer
 *
 * To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class AjBatchImageBuilder extends BatchImageBuilder {

	private AjState ajState;
	
	/**
	 * @param javaBuilder
	 */
	public AjBatchImageBuilder(JavaBuilder javaBuilder) {
		super(javaBuilder);
	}
	
	

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.internal.core.builder.AbstractImageBuilder#newCompiler()
	 */
	protected Compiler newCompiler() {
		ajState = new AjState();
		
		IMessageHandler weaverMessageHandler = new WeaverMessageHandler(javaBuilder.javaProject,this);
		
		// called once when the builder is initialized... can override if needed
		AjCompiler ajc =  new AjCompiler(
				nameEnvironment,
				DefaultErrorHandlingPolicies.proceedWithAllProblems(),
				javaBuilder.javaProject.getOptions(true),
				this,
				ProblemFactory.getProblemFactory(Locale.getDefault()),
				weaverMessageHandler,
				true);

		ajState.setWeaver(ajc.getWeaver());
		ajState.setWorld(ajc.getWorld());
		
		return ajc;		
	}			



	/* (non-Javadoc)
	 * @see org.eclipse.jdt.internal.core.builder.BatchImageBuilder#build()
	 */
	public void build() {
		ajState.prepareForNextBuild(null);
		super.build(); // calls cleanup too early for us...
		notifier.checkCancel();
		notifier.subTask("weaving"); //$NON-NLS-1$
		weaveAndGenerateClassFiles();
		reallyCleanUp();
	}
			
	public void weaveAndGenerateClassFiles()  {
		AjCompiler ajc = (AjCompiler)compiler;
		ajc.getWeaver().setProgressListener(new IProgressListener() {

			public void setText(String arg0) {
				System.err.println(arg0);
			}

			public void setProgress(double arg0) {
			}},0.5,0.5);
		
		for (Iterator i = ajState.addedClassFiles.iterator(); i.hasNext(); ) {
				UnwovenClassFile classFile = (UnwovenClassFile) i.next();
				((AjCompiler)compiler).getWeaver().addClassFile(classFile);
		}
		
		try {
			ajc.getWeaver().weave();
			ajc.getWeaver().dumpResourcesToOutPath();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.internal.core.builder.AbstractImageBuilder#cleanUp()
	 */
	protected void cleanUp() {
	  // not yet you don't
	}
	
	private void reallyCleanUp() {
		super.cleanUp();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.internal.compiler.ICompilerRequestor#acceptResult(org.eclipse.jdt.internal.compiler.CompilationResult)
	 */
	public void acceptResult(CompilationResult result) {
		super.acceptResult(result);
		outputClassFiles(result);
	}
	
	public void outputClassFiles(CompilationResult unitResult) {
		if (unitResult == null) return;
		
		String sourceFileName = new String(unitResult.fileName);
		if (!(unitResult.hasErrors())) {
			List unwovenClassFiles = new ArrayList();
			Enumeration classFiles = unitResult.compiledTypes.elements();
			while (classFiles.hasMoreElements()) {
				ClassFile classFile = (ClassFile) classFiles.nextElement();
				String filename = new String(classFile.fileName());
				filename = filename.replace('/', File.separatorChar) + ".class";
				
				File destinationPath = null;
				
				try {
					destinationPath = new File(ResourcesPlugin.getWorkspace().getRoot().getLocation().toOSString() + 
								javaBuilder.javaProject.getOutputLocation().toOSString());
				} catch (JavaModelException e) {
					e.printStackTrace();
				}
				
				if (destinationPath == null) {
					filename = new File(filename).getName();
					//filename = new File(extractDestinationPathFromSourceFile(unitResult), filename).getPath();
				} else {
					filename = new File(destinationPath, filename).getPath();
				}
				
				unwovenClassFiles.add(new UnwovenClassFile(filename, classFile.getBytes()));
			}
			ajState.noteClassesFromFile(unitResult, sourceFileName, unwovenClassFiles);
		} else {
			ajState.noteClassesFromFile(null, sourceFileName, Collections.EMPTY_LIST);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jdt.internal.core.builder.AbstractImageBuilder#createProblemFor(org.eclipse.core.resources.IResource, java.lang.String, java.lang.String)
	 */
	public void createProblemFor(  // raise visibility
			IResource resource,
			String message,
			String problemSeverity,
			int line) {
		try {
			IMarker marker = resource.createMarker(IJavaModelMarker.JAVA_MODEL_PROBLEM_MARKER);
			int severity = problemSeverity.equals(JavaCore.WARNING) ? IMarker.SEVERITY_WARNING : IMarker.SEVERITY_ERROR;

			marker.setAttributes(
					new String[] {IMarker.MESSAGE, IMarker.SEVERITY, IMarker.LINE_NUMBER},
					new Object[] {message, new Integer(severity), new Integer(line)});
		} catch (CoreException e) {
			throw internalException(e);
		}
	}
	
	
	private static class WeaverMessageHandler implements IMessageHandler {

		IJavaProject javaProject;
		AjBatchImageBuilder builder;
		
		WeaverMessageHandler(IJavaProject p, AjBatchImageBuilder b) {
			this.javaProject = p;
			this.builder = b;
		}
		
		public boolean handleMessage(IMessage msg) throws AbortException {
			IMessage.Kind kind = msg.getKind();
			if (kind == IMessage.INFO || kind == IMessage.DEBUG) return true;
			
			IResource onResource = javaProject.getProject();
			if (msg.getSourceLocation() != null) {
				File sourceFile = msg.getSourceLocation().getSourceFile();
				IResource target = javaProject.getProject().getParent().findMember(new Path(sourceFile.toString())); // descend?
				if (target != null) onResource = target;
			}

			String severity = JavaCore.WARNING;
			if (kind != IMessage.WARNING ) severity = JavaCore.ERROR;
			builder.createProblemFor(onResource, msg.getMessage(), severity, msg.getSourceLocation().getLine());
			
			return true;
		}

		public boolean isIgnoring(Kind arg0) {
			return true;
		}
		
		
	}

}
